/*
   Name:    "ShootOut.rex"

   Invoke:  "java com.ibm.bsf.Main -lang rexx -in ShootOut.rex"
                or
            "java com.ibm.bsf.Main            -in ShootOut.rex"
                or
            "rexxj ShootOut.rex"
                or
            "rexx ShootOut.rex"

   Purpose:  create a window with a choice-object and two text-fields and buttons;
             if the ok-button is pressed then popup another window and display what
             was chosen and entered.

   Needs:    IBM's "Bean Scripting Framework" (BSF)
             (cf. "http://oss.software.ibm.com/developerworks/projects/bsf")

             and the package "bsf4rexx"
             (cf. "http://nestroy.wi-inf.uni-essen.de/Forschung/rgf/Entwicklung.html")

   Date:    2001-04-21, 2003-01-23

   Author:   Rony G. Flatscher, WU Wien University
*/

if rxFuncQuery("BSF") then  /* no bsf4rexx support available yet, hence load Java */
do
   call rxFuncAdd "BsfLoadFuncs", "BSF4Rexx", "BsfLoadFuncs"
   call BsfLoadFuncs    /* load all BSF-functions                    */
   call BsfLoadJava     /* load the JVM and set up BSF4Rexx-support  */
end

if      BsfInvokedBy()=1 then say "This Rexx program was invoked by Java!"
else if BsfInvokedBy()=2 then say "This Rexx program was invoked by Rexx, JVM loaded by Rexx!"
else                          say "No JVM present, we got troubles ..."

        /* create labels                */
label1 =  BSF( 'registerBean', '',        'java.awt.Label', 'String', 'Your question?')
label2 =  BSF( 'registerBean', 'Label2',  'java.awt.Label', 'St'    , 'Well, consider this answer:' )

        /* get static values from class */
scrNone   = bsf('getStaticValue', 'java.awt.TextArea', 'SCROLLBARS_NONE')
scrVert   = bsf('getStaticValue', 'java.awt.TextArea', 'SCROLLBARS_VERTICAL_ONLY')

        /* create text area objects     */
taQuery   = bsf('registerBean', '', 'java.awt.TextArea', 'str', "", 'i',   5, 'i', 80, 'i', scrNone)
taAnswer  = bsf('registerBean', '', 'java.awt.TextArea', 'str', "" , 'i', 15, 'i', 80, 'i', scrVert)

        /* make the answer text area non-editable       */
call bsf 'invoke', taAnswer, 'setEditable', 'boolean', 0

        /* create the buttons           */
butAnswer = BSF( 'registerBean', '',      'java.awt.Button', 'String', ' Please answer me! ')
butLeave  = BSF( 'registerBean', '',      'java.awt.Button', 'String', ' Let us stop for dooday, o.k.? ' )

        /* create gridBagLayout and window      */
lay    = BSF( 'registerBean', '', 'java.awt.GridBagLayout')
layCon = BSF( 'registerBean', '', 'java.awt.GridBagConstraints')

        /* now define details of the GridBagLayout      */
call BSF 'setFieldValue', layCon, 'fill',      'I', bsf('getFieldValue', layCon, 'HORIZONTAL')
call BSF 'setFieldValue', layCon, 'gridwidth', 'I', bsf('getFieldValue', layCon, 'REMAINDER')
inset  = BSF( 'registerBean', '', 'java.awt.Insets', 'i', 5, 'i', 5, 'i', 5, 'i', 5)
call BSF 'setFieldValue', layCon, 'insets',     'o', inset

        /* register the constraints with the layout manager     */
call bsf 'invoke', lay, 'setConstraints', 'o', label1,    'o', layCon
call bsf 'invoke', lay, 'setConstraints', 'o', taQuery,   'o', layCon
call bsf 'invoke', lay, 'setConstraints', 'o', butAnswer, 'o', layCon
call bsf 'invoke', lay, 'setConstraints', 'o', label2,    'o', layCon
call bsf 'invoke', lay, 'setConstraints', 'o', taAnswer,  'o', layCon
call bsf 'invoke', lay, 'setConstraints', 'o', butLeave,  'o', layCon

        /* create the frame window                              */
Window = BSF( 'registerBean', '',      'java.awt.Frame', 'String', 'ShootOut!')

        /* now add controls to window, change some settings of it       */
call BSF 'invoke', Window, 'setLayout', 'Object', lay
call BSF 'invoke', Window, 'add',  'O', label1
call BSF 'invoke', Window, 'add',  'O', taQuery
call BSF 'invoke', Window, 'add',  'O', butAnswer
call BSF 'invoke', Window, 'add',  'O', Label2
call BSF 'invoke', Window, 'add',  'O', taAnswer
call BSF 'invoke', Window, 'add',  'O', butLeave

        /* pack the elements, show the window, bring it to the front    */
call BSF 'invoke', Window, 'pack'
call BSF 'invoke', Window, 'show'
call BSF 'invoke', Window, 'toFront'

        /* define the events we are interested in and the text to be sent to us */
call BSF 'addEventListener', Window,    'window', 'windowClosing', 'call BSF "exit"'
call BSF 'addEventListener', butAnswer, 'action', '', 'call answer taQuery, taAnswer'
call BSF 'addEventListener', butLeave , 'action', '', 'call BSF "exit"'

call get_answers        /* read answers from file       */

do forever      /* wait and fetch the text to be sent by Java           */
   a = bsf("pollEventText")     /* wait for an eventText to be sent     */
   interpret a                  /* execute as a Rexx program    */
   if result= "SHUTDOWN, REXX !" then leave     /* JVM will be shutdown */
end

exit

        /* read the file with the answers       */
get_answers: procedure expose answers.
   filename ="ShootAnswers.txt"

   i=1
   do i=1 to 9999 while chars(filename)> 0
      answers.i=linein(filename)        /* get line             */
   end                                  /* save number of items */
   answers.0=i-1
   return

        /* react to pressing the answer button  */
answer : procedure expose answers.
   parse arg taQuery, taAnswer          /* get names of TextAreas       */
   txt=bsf('invoke', taQuery, 'getText')        /* get text             */

   txt = strip( space(translate(txt, "  ", "0d0a"x))   ,'T')

   if right(txt, 1) = "?" then          /* a question in hand?          */
   do
      newLine = "0a0a"x
      tmp     =        newLine || "*** your question: ***" || newLine || txt
      tmp     = tmp || newLine || "--- my answer to you! ---" || newLine
      pick    = random(1, answers.0)    /* pick a number        */
      tmp     = tmp || answers.pick

      call bsf 'invoke', taAnswer, 'append',  'Str', tmp /* append to answer text area   */
      call bsf 'invoke', taQuery,  'setText', 'Str', ""
   end
   return

